package peer

import (
	"strconv"

	"cloud.o-forge.io/core/oc-lib/dbs"
	"cloud.o-forge.io/core/oc-lib/logs"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type peerMongoAccessor struct {
	utils.AbstractAccessor // AbstractAccessor contains the basic fields of an accessor (model, caller)
	overrideAuth           bool
}

// New creates a new instance of the peerMongoAccessor
func NewShallowAccessor() *peerMongoAccessor {
	return &peerMongoAccessor{
		overrideAuth: true,
		AbstractAccessor: utils.AbstractAccessor{
			Logger: logs.CreateLogger(tools.PEER.String()), // Create a logger with the data type
			Type:   tools.PEER,
		},
	}
}

func NewAccessor(request *tools.APIRequest) *peerMongoAccessor {
	return &peerMongoAccessor{
		overrideAuth: false,
		AbstractAccessor: utils.AbstractAccessor{
			Logger:  logs.CreateLogger(tools.PEER.String()), // Create a logger with the data type
			Request: request,
			Type:    tools.PEER,
		},
	}
}

func (wfa *peerMongoAccessor) ShouldVerifyAuth() bool {
	return !wfa.overrideAuth
}

/*
* Nothing special here, just the basic CRUD operations
 */

func (wfa *peerMongoAccessor) DeleteOne(id string) (utils.DBObject, int, error) {
	return utils.GenericDeleteOne(id, wfa)
}

func (wfa *peerMongoAccessor) UpdateOne(set utils.DBObject, id string) (utils.DBObject, int, error) {
	return utils.GenericUpdateOne(set.(*Peer), id, wfa, &Peer{})
}

func (wfa *peerMongoAccessor) StoreOne(data utils.DBObject) (utils.DBObject, int, error) {
	return utils.GenericStoreOne(data.(*Peer), wfa)
}

func (wfa *peerMongoAccessor) CopyOne(data utils.DBObject) (utils.DBObject, int, error) {
	return utils.GenericStoreOne(data, wfa)
}

func (dca *peerMongoAccessor) LoadOne(id string) (utils.DBObject, int, error) {
	return utils.GenericLoadOne[*Peer](id, func(d utils.DBObject) (utils.DBObject, int, error) {
		return d, 200, nil
	}, dca)
}

func (wfa *peerMongoAccessor) LoadAll(isDraft bool) ([]utils.ShallowDBObject, int, error) {
	return utils.GenericLoadAll[*Peer](func(d utils.DBObject) utils.ShallowDBObject {
		return d
	}, isDraft, wfa)
}

func (wfa *peerMongoAccessor) Search(filters *dbs.Filters, search string, isDraft bool) ([]utils.ShallowDBObject, int, error) {
	return utils.GenericSearch[*Peer](filters, search, wfa.getDefaultFilter(search),
		func(d utils.DBObject) utils.ShallowDBObject {
			return d
		}, isDraft, wfa)
}
func (a *peerMongoAccessor) getDefaultFilter(search string) *dbs.Filters {
	if i, err := strconv.Atoi(search); err == nil {
		return &dbs.Filters{
			Or: map[string][]dbs.Filter{ // search by name if no filters are provided
				"state": {{Operator: dbs.EQUAL.String(), Value: i}},
			},
		}
	} else {
		if search == "*" {
			search = ""
		}
		return &dbs.Filters{
			Or: map[string][]dbs.Filter{ // search by name if no filters are provided
				"abstractobject.name": {{Operator: dbs.LIKE.String(), Value: search}},
				"url":                 {{Operator: dbs.LIKE.String(), Value: search}},
			},
		}
	}
}
