package resources

import (
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type ResourceSet struct {
	Datas       []string `bson:"datas,omitempty" json:"datas,omitempty"`
	Storages    []string `bson:"storages,omitempty" json:"storages,omitempty"`
	Processings []string `bson:"processings,omitempty" json:"processings,omitempty"`
	Computes    []string `bson:"computes,omitempty" json:"computes,omitempty"`
	Workflows   []string `bson:"workflows,omitempty" json:"workflows,omitempty"`

	DataResources       []*DataResource       `bson:"-" json:"data_resources,omitempty"`
	StorageResources    []*StorageResource    `bson:"-" json:"storage_resources,omitempty"`
	ProcessingResources []*ProcessingResource `bson:"-" json:"processing_resources,omitempty"`
	ComputeResources    []*ComputeResource    `bson:"-" json:"compute_resources,omitempty"`
	WorkflowResources   []*WorkflowResource   `bson:"-" json:"workflow_resources,omitempty"`
}

func (r *ResourceSet) Clear() {
	r.DataResources = nil
	r.StorageResources = nil
	r.ProcessingResources = nil
	r.ComputeResources = nil
	r.WorkflowResources = nil
}

func (r *ResourceSet) Fill(request *tools.APIRequest) {
	r.Clear()
	for k, v := range map[utils.DBObject][]string{
		(&DataResource{}):       r.Datas,
		(&ComputeResource{}):    r.Computes,
		(&StorageResource{}):    r.Storages,
		(&ProcessingResource{}): r.Processings,
		(&WorkflowResource{}):   r.Workflows,
	} {
		for _, id := range v {
			d, _, e := k.GetAccessor(request).LoadOne(id)
			if e == nil {
				switch k.(type) {
				case *DataResource:
					r.DataResources = append(r.DataResources, d.(*DataResource))
				case *ComputeResource:
					r.ComputeResources = append(r.ComputeResources, d.(*ComputeResource))
				case *StorageResource:
					r.StorageResources = append(r.StorageResources, d.(*StorageResource))
				case *ProcessingResource:
					r.ProcessingResources = append(r.ProcessingResources, d.(*ProcessingResource))
				case *WorkflowResource:
					r.WorkflowResources = append(r.WorkflowResources, d.(*WorkflowResource))
				}
			}
		}
	}
}

type ItemResource struct {
	Data       *DataResource       `bson:"data,omitempty" json:"data,omitempty"`
	Processing *ProcessingResource `bson:"processing,omitempty" json:"processing,omitempty"`
	Storage    *StorageResource    `bson:"storage,omitempty" json:"storage,omitempty"`
	Compute    *ComputeResource    `bson:"compute,omitempty" json:"compute,omitempty"`
	Workflow   *WorkflowResource   `bson:"workflow,omitempty" json:"workflow,omitempty"`
}
