package workspace

import (
	"cloud.o-forge.io/core/oc-lib/models/collaborative_area/shallow_collaborative_area"
	"cloud.o-forge.io/core/oc-lib/models/resources"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

// Workspace is a struct that represents a workspace
type Workspace struct {
	utils.AbstractObject         // AbstractObject contains the basic fields of an object (id, name)
	resources.ResourceSet        // ResourceSet contains the resources of the workspace (data, compute, processing, storage, workflow)
	IsContextual          bool   `json:"is_contextual" bson:"is_contextual" default:"false"` // IsContextual is a flag that indicates if the workspace is contextual
	Active                bool   `json:"active" bson:"active" default:"false"`               // Active is a flag that indicates if the workspace is active
	Shared                string `json:"shared,omitempty" bson:"shared,omitempty"`           // Shared is the ID of the shared workspace
}

func (d *Workspace) GetAccessor(request *tools.APIRequest) utils.Accessor {
	return NewAccessor(request) // Create a new instance of the accessor
}

func (ao *Workspace) VerifyAuth(request *tools.APIRequest) bool {
	if ao.Shared != "" {
		shared, code, _ := shallow_collaborative_area.NewAccessor(request).LoadOne(ao.Shared)
		if code != 200 || shared == nil {
			return false
		}
		return shared.VerifyAuth(request)
	}
	return ao.AbstractObject.VerifyAuth(request)
}
